@@ -1,7 +1,7 @@
 package Perinci::Examples::FileStream;
 
-our $DATE = '2015-01-11'; # DATE
-our $VERSION = '0.46'; # VERSION
+our $DATE = '2015-01-17'; # DATE
+our $VERSION = '0.47'; # VERSION
 
 use 5.010;
 use strict;
@@ -38,12 +38,9 @@ $SPEC{read_file} = {
 
 This function demonstrate output streaming of bytes.
 
-To do output streaming, on the function side, you just return an opened
-filehandle glob which can be read from, or an `IO::Handle` object (anything that
-supports `getline()` or `getitem()` method) or an array (usually a tied array).
-
-`Perinci::CmdLine` supports this and will read items/lines from the
-handle/object/tied array until exhausted, and then output them.
+To do output streaming, on the function side, you just return a coderef which
+can be called by caller (e.g. CLI framework `Perinci::CmdLine`) to read data
+from. Code must return data or undef to signify exhaustion.
 
 This works over remote (HTTP) too, because output streaming is supported by
 `Riap::HTTP` (version 1.2) and `Perinci::Access::HTTP::Client`. Streams are
@@ -59,7 +56,7 @@ sub read_file {
 
     open my($fh), "<", $path or return [500, "Can't open '$path': $!"];
 
-    [200, "OK", $fh, {stream=>1}];
+    [200, "OK", sub { ~~<$fh> }, {stream=>1}];
 }
 
 $SPEC{write_file} = {
@@ -72,15 +69,15 @@ To do input streaming, on the function side, you just specify one your args with
 the `stream` property set to true (`stream => 1`). In this example, the
 `content` argument is set to streaming.
 
-If you run the function through `Perinci::CmdLine`, you'll get a filehandle
-instead of the actual value. You can then iterate the content from this
-filehandle. This currently works for local functions only. As of this writing,
-`Riap::HTTP` protocol does not support input streaming. It supports partial
-input though (see the documentation on how this works) and theoretically
-streaming can be emulated by client library using partial input. However, client
-like `Perinci::Access::HTTP::Client` does not yet support this.
+If you run the function through `Perinci::CmdLine`, you'll get a coderef instead
+of the actual value. You can then repeatedly call the code to read data. This
+currently works for local functions only. As of this writing, `Riap::HTTP`
+protocol does not support input streaming. It supports partial input though (see
+the documentation on how this works) and theoretically streaming can be emulated
+by client library using partial input. However, client like
+`Perinci::Access::HTTP::Client` does not yet support this.
 
-Note that the argument's schema is still `buf*`, not `obj*`.
+Note that the argument's schema is still `buf*`, not `code*`.
 
 Note: This function overwrites existing file.
 
@@ -88,7 +85,8 @@ _
     args => {
         path => {schema=>'str*', req=>1, pos=>0},
         content => {schema=>'buf*', req=>1, pos=>1, stream=>1,
-                    cmdline_src=>'stdin_or_files'},
+                    cmdline_src=>'stdin_or_files',
+                },
     },
 };
 sub write_file {
@@ -101,9 +99,10 @@ sub write_file {
     my $content = $args{content};
     my $written = 0;
     if (ref($content)) {
-        my $fh = $content;
-        local $/ = \(64*1024);
-        while (<$fh>) { print $fh $_; $written += length($_) }
+        local $_;
+        while (defined($_ = $content->())) {
+            print $fh $_; $written += length($_);
+        }
     } else {
         print $fh $content;
         $written += length($content);
@@ -136,9 +135,10 @@ sub append_file {
     my $content = $args{content};
     my $written = 0;
     if (ref($content)) {
-        my $fh = $content;
-        local $/ = \(64*1024);
-        while (<$fh>) { print $fh $_; $written += length($_) }
+        local $_;
+        while (defined($_ = $content->())) {
+            print $fh $_; $written += length($_);
+        }
     } else {
         print $fh $content;
         $written += length($content);
@@ -162,7 +162,7 @@ Perinci::Examples::FileStream - Examples for reading/writing files (using stream
 
 =head1 VERSION
 
-This document describes version 0.46 of Perinci::Examples::FileStream (from Perl distribution Perinci-Examples), released on 2015-01-11.
+This document describes version 0.47 of Perinci::Examples::FileStream (from Perl distribution Perinci-Examples), released on 2015-01-17.
 
 =head1 DESCRIPTION
 
@@ -210,12 +210,9 @@ Return value:  (any)
 
 This function demonstrate output streaming of bytes.
 
-To do output streaming, on the function side, you just return an opened
-filehandle glob which can be read from, or an C<IO::Handle> object (anything that
-supports C<getline()> or C<getitem()> method) or an array (usually a tied array).
-
-C<Perinci::CmdLine> supports this and will read items/lines from the
-handle/object/tied array until exhausted, and then output them.
+To do output streaming, on the function side, you just return a coderef which
+can be called by caller (e.g. CLI framework C<Perinci::CmdLine>) to read data
+from. Code must return data or undef to signify exhaustion.
 
 This works over remote (HTTP) too, because output streaming is supported by
 C<Riap::HTTP> (version 1.2) and C<Perinci::Access::HTTP::Client>. Streams are
@@ -249,15 +246,15 @@ To do input streaming, on the function side, you just specify one your args with
 the C<stream> property set to true (C<< stream =E<gt> 1 >>). In this example, the
 C<content> argument is set to streaming.
 
-If you run the function through C<Perinci::CmdLine>, you'll get a filehandle
-instead of the actual value. You can then iterate the content from this
-filehandle. This currently works for local functions only. As of this writing,
-C<Riap::HTTP> protocol does not support input streaming. It supports partial
-input though (see the documentation on how this works) and theoretically
-streaming can be emulated by client library using partial input. However, client
-like C<Perinci::Access::HTTP::Client> does not yet support this.
+If you run the function through C<Perinci::CmdLine>, you'll get a coderef instead
+of the actual value. You can then repeatedly call the code to read data. This
+currently works for local functions only. As of this writing, C<Riap::HTTP>
+protocol does not support input streaming. It supports partial input though (see
+the documentation on how this works) and theoretically streaming can be emulated
+by client library using partial input. However, client like
+C<Perinci::Access::HTTP::Client> does not yet support this.
 
-Note that the argument's schema is still C<buf*>, not C<obj*>.
+Note that the argument's schema is still C<buf*>, not C<code*>.
 
 Note: This function overwrites existing file.
 
@@ -288,7 +285,7 @@ Please visit the project's homepage at L<https://metacpan.org/release/Perinci-Ex
 
 =head1 SOURCE
 
-Source repository is at L<https://github.com/sharyanto/perl-Perinci-Examples>.
+Source repository is at L<https://github.com/perlancar/perl-Perinci-Examples>.
 
 =head1 BUGS
 
